@@ -51,11 +51,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include	"eachfile_p.h"
 #include	"xml_p.h"
 #include	"dbinfo.h"
+#include	"show.h"
 #ifdef		ELC_FIND_LEAKS
 #include	"leakfind.h"
 #endif
 #define		ALGO_NAME	"SHA-1"
 
+
 typedef struct fileinfo {	/* database file information */
   dbinfo	dbinf;
   char		*path;		/* null-terminated */
@@ -133,13 +135,13 @@ static void show_diff_xml_long(FILE *out, char T, const char *type,
     xml_end_print(out, type);
 }
 
-static void show_diff_lines_long(FILE *out, char T, const char *type,
+static void show_diff_long(FILE *out, char T, const char *type,
 				 long old, long new)
 {
     fprintf(out, "%c(%ld:%ld) ", T, old, new);
 }
 
-static void show_diff_lines_time(FILE *out, char T, const char *type,
+static void show_diff_time(FILE *out, char T, const char *type,
 				 time_t old, time_t new)
 {
     const int	buf_max	 = 16;	/* 20001212-010101 + null */
@@ -174,56 +176,76 @@ static void show_diff_xml_octal(FILE *out, char T, const char *type,
     xml_end_print(out, type);
 }
 
-static void show_diff_lines_octal(FILE *out, char T, const char *type,
+static void show_diff_octal(FILE *out, char T, const char *type,
 				  unsigned long old, unsigned long new)
 {
     fprintf(out, "%c(%lo:%lo) ", T, old, new);
 }
 
+static void show_diff_perms(FILE *out, mode_t old, mode_t new)
+{
+    fputs("p(", out);
+    if (S_ISLNK(old)) /* for a symlink ... */
+      fputs("sym", out); /* display "sym" instead of "777" */
+    else
+      fprintf(out, "%o", old & PERM_MASK);
+
+    putc(':', out);
+
+    if (S_ISLNK(new))
+      fputs("sym", out);
+    else
+      fprintf(out, "%o) ", new & PERM_MASK);
+}
+
 static void show_diffs(options *opts,
 		       const char *path, unsigned long diffs,
 		       const struct stat *sa, const struct stat *sb)
 {
-    void (*show_long)(FILE *out, char T, const char *type, long old, long new);
-    void (*show_time)(FILE *out, char T, const char *type,
+    /* these function pointers use "sho" for "show" to avoid confusion
+     * with functions from show.c */
+    void (*sho_long)(FILE *out, char T, const char *type, long old, long new);
+    void (*sho_time)(FILE *out, char T, const char *type,
 		      time_t old, time_t new);
-    void (*show_oct)(FILE *out, char T, const char *type,
+    void (*sho_oct)(FILE *out, char T, const char *type,
 		     unsigned long old, unsigned long new);
 
-    /* for showing the last twelve bits (four octal digits) */
-    const unsigned	perm_mask	 = 07777;
-
     if (opts->output == OUTPUT_XML) {
-      show_long	 = show_diff_xml_long;
-      show_time	 = show_diff_xml_long;
-      show_oct	 = show_diff_xml_octal;
+      sho_long	 = show_diff_xml_long;
+      sho_time	 = show_diff_xml_long;
+      sho_oct	 = show_diff_xml_octal;
       XML_CHANGE_START_PRINT(stdout, "stat", path);
     } else {
-      show_long	 = show_diff_lines_long;
-      show_time	 = show_diff_lines_time;
-      show_oct	 = show_diff_lines_octal;
+      sho_long	 = show_diff_long;
+      sho_time	 = show_diff_time;
+      sho_oct	 = show_diff_octal;
       fprintf(stdout, "changed: %s   ", path);
     }
 
     if (diffs & RULE_INODE)
-      show_long(stdout, 'i', "inode", sa->st_ino, sb->st_ino);
-    if (diffs & RULE_PERMS)
-      show_oct(stdout, 'p', "permissions",
-	       sa->st_mode & perm_mask, sb->st_mode & perm_mask);
+      sho_long(stdout, 'i', "inode", sa->st_ino, sb->st_ino);
+    if (diffs & RULE_PERMS) {
+      if (opts->output == OUTPUT_XML) {
+	sho_oct(stdout, 'p', "permissions",
+		 sa->st_mode & PERM_MASK, sb->st_mode & PERM_MASK);
+      } else {
+	show_diff_perms(stdout, sa->st_mode, sb->st_mode);
+      }
+    }
     if (diffs & RULE_NLINK)
-      show_long(stdout, 'l', "nlinks", sa->st_nlink, sb->st_nlink);
+      sho_long(stdout, 'l', "nlinks", sa->st_nlink, sb->st_nlink);
     if (diffs & RULE_UID)
-      show_long(stdout, 'u', "uid", sa->st_uid, sb->st_uid);
+      sho_long(stdout, 'u', "uid", sa->st_uid, sb->st_uid);
     if (diffs & RULE_GID)
-      show_long(stdout, 'g', "gid", sa->st_gid, sb->st_gid);
+      sho_long(stdout, 'g', "gid", sa->st_gid, sb->st_gid);
     if (diffs & RULE_SIZE)
-      show_long(stdout, 'z', "size", sa->st_size, sb->st_size);
+      sho_long(stdout, 'z', "size", sa->st_size, sb->st_size);
     if (diffs & RULE_ATIME)
-      show_time(stdout, 'a', "access_time", sa->st_atime, sb->st_atime);
+      sho_time(stdout, 'a', "access_time", sa->st_atime, sb->st_atime);
     if (diffs & RULE_MTIME)
-      show_time(stdout, 'm', "modification_time", sa->st_mtime, sb->st_mtime);
+      sho_time(stdout, 'm', "modification_time", sa->st_mtime, sb->st_mtime);
     if (diffs & RULE_CTIME)
-      show_time(stdout, 'c', "change_time", sa->st_ctime, sb->st_ctime);
+      sho_time(stdout, 'c', "change_time", sa->st_ctime, sb->st_ctime);
 
     if (opts->output == OUTPUT_XML)
       XML_END_PRINT(stdout, "change");
@@ -285,7 +307,7 @@ static void report_2regfile(options *opts, const char *path)
 	putc('\n', stdout);
 	break;
       case OUTPUT_LINES:
-	printf("changed: %s became regular file\n", path);
+	printf("changed: %s   became regular file\n", path);
 	break;
       default:
 	abort();		/* shouldn't happen */
@@ -353,25 +375,56 @@ static void report_differences(options *opts, fileinfo *currinf)
     report_stat_differences(opts, currinf, &old);
 }
 
+static void show_xml_long(FILE *out, const char *type, long val)
+{
+    xml_start_print(out, type);
+    fprintf(out, "%ld", val);
+    xml_end_print(out, type);
+}
+static void show_xml_octal(FILE *out, const char *type, unsigned long val)
+{
+    xml_start_print(out, type);
+    fprintf(out, "%lo", val);
+    xml_end_print(out, type);
+}
+
+
+/* similar to aux/show.c: show_entry */
 static void report_newfile(options *opts, fileinfo *inf)
 {
     char	*path	 = inf->path;
+    struct stat	*s	 = &inf->dbinf.stat;
 
     switch (opts->output) {
       case OUTPUT_XML:
 	XML_CHANGE_START_PRINT(stdout, "newfile", path);
+	show_xml_octal(stdout, "permissions", s->st_mode & PERM_MASK);
+	show_xml_long(stdout, "uid", s->st_uid);
+	show_xml_long(stdout, "gid", s->st_gid);
+	show_xml_long(stdout, "size", s->st_size);
+	show_xml_long(stdout, "modification_time", s->st_mtime);
 	XML_END_PRINT(stdout, "change");
-	putc('\n', stdout);
 	break;
       case OUTPUT_LINES:
 	/* use some extra spaces for alignment
 	 * with "missing:" and "changed:" */
-	printf("new:     %s\n", path); /* this file wasn't in known db */
+	printf("new:     %s   ", path); /* this file wasn't in known db */
+
+	if (S_ISLNK(s->st_mode)) /* for a symlink ... */
+	  fputs("p(sym) ", stdout); /* display "sym" instead of "777" */
+	else
+	  show_octal(stdout, 'p', s->st_mode & PERM_MASK);
+
+	show_long(stdout, 'u', s->st_uid);
+	show_long(stdout, 'g', s->st_gid);
+	show_long(stdout, 'z', s->st_size);
+	show_time(stdout, 'm', s->st_mtime);
 	break;
       default:
 	abort();		/* this shouldn't happen */
 	break;
     }
+    putc('\n', stdout);
 }    
 
 inline static void do_check(options *opts, fileinfo *inf)
